using System;
using System.ComponentModel ;
using System.Collections ;
namespace ICustomTypeSample
{
	/// <summary>
	/// Summary description for BindableCollection.
	/// </summary>
	public class BookCollection: IList   
	{
		
		ArrayList innerArray;
			
		/// <summary>
		/// Initializes a new instance of the BookCollection class.
		/// </summary>
		public BookCollection()
		{
			innerArray = new ArrayList();
		}

		#region Memebers and methods
		/// <summary>
		/// Gets the number of elements in the BookCollection.
		/// </summary>
		/// <value>
		/// The number of elements contained in the BookCollection.
		/// </value>
		public int Count
		{
			get { return innerArray.Count; }
		}

		/// <summary>
		/// Gets a value indicating whether the BookCollection has a fixed size.
		/// </summary>
		/// <value>
		/// true if the BookCollection has a fixed size; otherwise, false.
		/// </value>
		public bool IsFixedSize
		{
			get { return false; }
		}
			
		/// <summary>
		/// Gets a value indicating whether the BookCollection is read-only.
		/// </summary>
		public bool IsReadOnly
		{
			get { return false; }
		}

		/// <summary>
		/// Gets a value indicating whether access to the collection is synchronized (thread-safe).
		/// </summary>
		/// <value>
		/// true if access to the BookCollection is synchronized (thread-safe); otherwise, false.
		/// </value>
		public bool IsSynchronized
		{
			get { return false; }
		}

		/// <summary>
		/// Gets an object that can be used to synchronize access to the collection.
		/// </summary>
		/// <value>
		/// An object that can be used to synchronize access to the collection.
		/// </value>
		object ICollection.SyncRoot
		{
			get { return null; }
		}

		/// <summary>
		/// Gets or sets the element at the specified index.
		/// In C#, this property is the indexer for the BookCollection class.
		/// </summary>
		/// <param name="index">The zero-based index of the element to get or set.</param>
		/// <value>
		/// The element at the specified index.
		/// </value>
		public Book this[int index]
		{
			get { return (Book)innerArray[index]; }
			set { innerArray[index] = value; }
		}

		/// <summary>
		/// Adds a Book to the end of the BookCollection.
		/// </summary>
		/// <param name="value">The Book to be added to the end of the BookCollection.</param>
		/// <returns>The BookCollection index at which the value has been added.</returns>
		public int Add(Book value)
		{
			int index = innerArray.Add(value);

			return index;
		}

		/// <summary>
		/// Adds the elements of an array of Book objects to the end of the BookCollection.
		/// </summary>
		/// <param name="array">The Book array whose elements should be added to the end of the
		/// BookCollection.</param>
		public void AddRange(Book[] array)
		{
			innerArray.AddRange(array);
		}

		/// <summary>
		/// Removes all elements from the BookCollection.
		/// </summary>
		public void Clear()
		{
			innerArray.Clear();
		}

		/// <summary>
		/// Determines whether a Book is in the BookCollection.
		/// </summary>
		/// <param name="item">The Book to locate in the BookCollection. The element to locate
		/// can be a null reference (Nothing in Visual Basic).</param>
		/// <returns>true if item is found in the BookCollection; otherwise, false.</returns>
		public bool Contains(Book item)
		{
			return innerArray.Contains(item);
		}

		/// <summary>
		/// Determines whether a Book with the specified name is in the BookCollection.
		/// </summary>
		/// <param name="name">The name of the Book to locate in the BookCollection.</param>
		/// <returns>true if item is found in the BookCollection; otherwise, false.</returns>
		public bool Contains(string name)
		{
			foreach(Book spec in innerArray)
				if(spec.Name == name)
					return true;

			return false;
		}

		/// <summary>
		/// Copies the entire BookCollection to a compatible one-dimensional Array, starting at the
		/// beginning of the target array.
		/// </summary>
		/// <param name="array">The one-dimensional Array that is the destination of the elements copied
		/// from BookCollection. The Array must have zero-based indexing.</param>
		public void CopyTo(Book[] array)
		{
			innerArray.CopyTo(array);
		}

		/// <summary>
		/// Copies the BookCollection or a portion of it to a one-dimensional array.
		/// </summary>
		/// <param name="array">The one-dimensional Array that is the destination of the elements copied
		/// from the collection.</param>
		/// <param name="index">The zero-based index in array at which copying begins.</param>
		public void CopyTo(Book[] array, int index)
		{
			innerArray.CopyTo(array, index);
		}

		/// <summary>
		/// Returns an enumerator that can iterate through the BookCollection.
		/// </summary>
		/// <returns>An IEnumerator for the entire BookCollection.</returns>
		public IEnumerator GetEnumerator()
		{
			return innerArray.GetEnumerator();
		}

		/// <summary>
		/// Searches for the specified Book and returns the zero-based index of the first
		/// occurrence within the entire BookCollection.
		/// </summary>
		/// <param name="value">The Book to locate in the BookCollection.</param>
		/// <returns>The zero-based index of the first occurrence of value within the entire BookCollection,
		/// if found; otherwise, -1.</returns>
		public int IndexOf(Book value)
		{
			return innerArray.IndexOf(value);
		}

		/// <summary>
		/// Searches for the Book with the specified name and returns the zero-based index of
		/// the first occurrence within the entire BookCollection.
		/// </summary>
		/// <param name="name">The name of the Book to locate in the BookCollection.</param>
		/// <returns>The zero-based index of the first occurrence of value within the entire BookCollection,
		/// if found; otherwise, -1.</returns>
		public int IndexOf(string name)
		{
			int i = 0;

			foreach(Book spec in innerArray)
			{
				if(spec.Name == name)
					return i;

				i++;
			}

			return -1;
		}

		/// <summary>
		/// Inserts a Book object into the BookCollection at the specified index.
		/// </summary>
		/// <param name="index">The zero-based index at which value should be inserted.</param>
		/// <param name="value">The Book to insert.</param>
		public void Insert(int index, Book value)
		{
			innerArray.Insert(index, value);
		}

		/// <summary>
		/// Removes the first occurrence of a specific object from the BookCollection.
		/// </summary>
		/// <param name="obj">The Book to remove from the BookCollection.</param>
		public void Remove(Book obj)
		{
			innerArray.Remove(obj);
		}

		/// <summary>
		/// Removes the property with the specified name from the BookCollection.
		/// </summary>
		/// <param name="name">The name of the Book to remove from the BookCollection.</param>
		public void Remove(string name)
		{
			int index = IndexOf(name);
			RemoveAt(index);
		}

		/// <summary>
		/// Removes the object at the specified index of the BookCollection.
		/// </summary>
		/// <param name="index">The zero-based index of the element to remove.</param>
		public void RemoveAt(int index)
		{
			innerArray.RemoveAt(index);
		}

		/// <summary>
		/// Copies the elements of the BookCollection to a new Book array.
		/// </summary>
		/// <returns>A Book array containing copies of the elements of the BookCollection.</returns>
		public Book[] ToArray()
		{
			return (Book[])innerArray.ToArray(typeof(Book));
		}
		#endregion

		#region Explicit interface implementations for ICollection and IList
		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		void ICollection.CopyTo(Array array, int index)
		{
			CopyTo((Book[])array, index);
		}

		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		int IList.Add(object value)
		{
			return Add((Book)value);
		}

		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		bool IList.Contains(object obj)
		{
			return Contains((Book)obj);
		}

		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		object IList.this[int index]
		{
			get
			{
				return ((BookCollection)this)[index];
			}
			set
			{
				((BookCollection)this)[index] = (Book)value;
			}
		}

		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		int IList.IndexOf(object obj)
		{
			return IndexOf((Book)obj);
		}

		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		void IList.Insert(int index, object value)
		{
			Insert(index, (Book)value);
		}

		/// <summary>
		/// This member supports the .NET Framework infrastructure and is not intended to be used directly from your code.
		/// </summary>
		void IList.Remove(object value)
		{
			Remove((Book)value);
		}
		#endregion


	}
}
